# TODO: Remove previous uploads

module Rise::Uploadable
  def self.included(base)
    base.extend ClassMethods
  end
  
  module ClassMethods
    attr_accessor :cache
  end
  
  attr_accessor :remove
  attr_reader :original_filename
  
  def filename
    original_filename
  end
  
  # TODO: test
  def identifier
    read_identifier
  end
  
  def upload=(file)
    return unless upload?(file)
    @cache_prefix = Rise.generate_cache_prefix unless cached?
    self.original_filename = determine_filename(file)
    @file = self.class.cache.put(file, path: cache_path)
  end
  
  def cache
    File.join(@cache_prefix, original_filename) if cached?
  end
  
  def cache=(identifier)
    return if identifier.nil? || identifier.empty?
    retrieve_from_cache!(identifier) unless cached?
  end
  
  def cached?
    !!@cache_prefix
  end

  def store!
    if remove?
      remove!
    elsif cached?
      stored = @file.copy(path: store_path, store: self.class.store)
      @file.destroy
      @file = stored
      @cache_prefix = nil
    end
  end
  
  # TODO: test
  def remove?
    ['1', true].include?(remove)
  end
  
  # TODO: test
  def remove!
    file.destroy if file
    @file, @cache_prefix, self.original_filename = nil
  end
  
  # TODO: test
  def cache_dir
    # Optionally override in destination class.
  end
  
  # TODO: test
  def store_dir
    # Optionally override in destination class.
  end
  
  # TODO: test
  def retrieve_from_store!(identifier)
    return if identifier.nil? || identifier.empty?
    @cache_prefix = nil
    #self.original_filename = identifier
    @file = self.class.store.get(store_path(identifier))
  rescue Stow::NotFoundError
    raise Rise::NotFoundError, "Missing file: '#{identifier}'"
  end
  
  # TODO: test
  def retrieve_from_cache!(identifier)
    @cache_prefix, self.original_filename = identifier.to_s.split('/', 2)
    raise Rise::InvalidCachePrefix unless @cache_prefix =~ Rise.cache_prefix_regexp
    @file = self.class.cache.get(cache_path)
  rescue Stow::NotFoundError
    raise Rise::NotFoundError, "Missing cache file: '#{identifier}'"
  end
  
  # TODO: test
  def file
    @file || retrieve_from_store!(read_identifier)
  end
  
  protected
  
  # TODO: test
  def read_identifier
    # Does nothing by default but ActiveModel would do:
    #   model.read_attribute :file
  end
  
  # TODO: test
  # before_save
  def write_identifier
    # Does nothing by default but ActiveModel would do:
    #   return unless cached? || remove?
    #   model.write_attribute :file, remove? ? '' : filename
  end
  
  private
  
  def upload?(file)
    if file.nil? || file.respond_to?(:empty?) && file.empty?
      false
    elsif file.respond_to?(:read)
      file.respond_to?(:path) && File.file?(file.path) || file.size > 0
    else
      raise Rise::MultipartError
    end
  end

  def cache_path(name = cache)
    File.join([cache_dir, name].compact)
  end
  
  def store_path(name = filename)
    File.join([store_dir, name].compact)
  end

  def original_filename=(filename)
    raise Rise::InvalidFilename if filename =~ Rise.sanitize_regexp
    @original_filename = filename
  end

  def determine_filename(file)
    name = if file.respond_to?(:original_filename)
      file.original_filename
    elsif file.respond_to?(:path) && !(file.path.nil? || file.path.empty?)
      File.basename(File.expand_path(file.path))
    end
    sanitize(name)
  end
  
  def sanitize(name)
    name = File.basename(name.to_s.gsub('\\', '/')) # IE
    name.gsub!(Rise.sanitize_regexp, '-')
    name.sub!(/\A(|[\.-]+)\z/, 'untitled')
    name.squeeze('-')
  end
end